# AutoTransform
# Large scale, component based code modification library
#
# Licensed under the MIT License <http://opensource.org/licenses/MIT>
# SPDX-License-Identifier: MIT
# Copyright (c) 2022-present Nathan Rockenbach <http://github.com/nathro>

# @black_format

"""Filters are used by AutoTransform to take Items provided by Inputs and check them for
eligibility. Items that pass Filters will get passed along to Batchers."""
