# AutoTransform
# Large scale, component based code modification library
#
# Licensed under the MIT License <http://opensource.org/licenses/MIT>
# SPDX-License-Identifier: MIT
# Copyright (c) 2022-present Nathan Rockenbach <http://github.com/nathro>

# @black_format

"""AutoTransform is a framework for defining and running code transformations.
It can be used by organizations looking to update/change/cleanup their codebases
as well as by library authors to provide transformations to end users that can
be used to update code for changes to the library (i.e. moving to new APIs).

For more information, see the README.
"""
