# AutoTransform
# Large scale, component based code modification library
#
# Licensed under the MIT License <http://opensource.org/licenses/MIT>
# SPDX-License-Identifier: MIT
# Copyright (c) 2022-present Nathan Rockenbach <http://github.com/nathro>

# @black_format

"""Validators take in a Batch that has undergone transformation and validate that the
changes do not produce any issues/errors. They may run test cases, check linting, perform
type checks, or some other process."""
