# AutoTransform
# Large scale, component based code modification library
#
# Licensed under the MIT License <http://opensource.org/licenses/MIT>
# SPDX-License-Identifier: MIT
# Copyright (c) 2022-present Nathan Rockenbach <http://github.com/nathro>

# @black_format

"""Batchers take filtered Items and separate them in to logical groupings
with associated metadata that can be acted on independently. Each Batch
represents a single Change (i.e. a Pull Request)."""
