# AutoTransform
# Large scale, component based code modification library
#
# Licensed under the MIT License <http://opensource.org/licenses/MIT>
# SPDX-License-Identifier: MIT
# Copyright (c) 2022-present Nathan Rockenbach <http://github.com/nathro>

# @black_format

"""Steps are used by the Manager to determine how to handle outstanding Changes
from AutoTransform. Merging accepted changes, abandoning rejected ones, updating
stale one, are all examples of potential steps."""
